@@ -34,7 +34,7 @@
 
 #include <stdlib.h>
 
-#include "global.h"
+#include "xst.h"
 
 #include "transfer.h"
 #include "e-table.h"
@@ -52,10 +52,6 @@ static XstDialogSignal signals[] = {
 	{ NULL }
 };
 
-static void set_access_sensitivity (void)
-{
-}
-
 static void
 update_complexity (void)
 {
@@ -101,7 +97,7 @@ main (int argc, char *argv[])
 
 	xst_dialog_thaw (tool->main_dialog);
 	
-	xst_tool_main (tool);
+	xst_tool_main (tool, FALSE);
 	
 	return 0;
 }
