@@ -28,6 +28,7 @@
 #include "xst-report-hook.h"
 #include "xst-platform.h"
 #include "xst-ui.h"
+#include "xst-su.h"
 
 #include <gnome.h>
 #include <parser.h>
@@ -35,6 +36,7 @@
 
 #include <stdio.h>
 #include <fcntl.h>
+#include <errno.h>
 
 #define XST_DEBUG 1
 
@@ -60,6 +62,8 @@ static gboolean report_finished_cb        (XstTool *tool, XstReportLine *rline);
 
 static void report_dispatch (XstTool *tool);
 
+static gint xst_tool_compare_platforms (XstPlatform *a, XstPlatform *b);
+
 static XstReportHookEntry common_report_hooks[] = {
 	{ "end",              report_finished_cb,         XST_REPORT_HOOK_LOADSAVE, TRUE  },
 	{ "platform_list",    platform_add_supported_cb,  XST_REPORT_HOOK_LOAD,     TRUE  }, 
@@ -69,6 +73,8 @@ static XstReportHookEntry common_report_hooks[] = {
 	{ 0,                  NULL,                       -1,                       FALSE }
 };
 
+static gchar *location_id;    /* Location in which we are editing */
+
 /* --- Report hook callbacks --- */
 
 static void
@@ -93,11 +99,17 @@ platform_list_unselect_row_cb (GtkCList *clist, gint row, gint column, GdkEvent
 	gtk_widget_set_sensitive (tool->platform_ok_button, FALSE);
 }
 
+static void
+platform_dialog_close_cb (GtkWidget *dialog, XstTool *tool)
+{
+	exit (0);
+}
+
 static void
 platform_unsupported_clicked_cb (GtkWidget *widget, gint ret, XstTool *tool)
 {
 	if (ret == -1 || ret == 1)
-		exit (1);
+		exit (0);
 
 	/* Locate the selected platform and set it up as the current one. Prepare to
 	 * invoke the backend again, this time with the current platform specified. */
@@ -117,6 +129,11 @@ platform_unsupported_clicked_cb (GtkWidget *widget, gint ret, XstTool *tool)
 	tool->run_again = TRUE;
 
 	report_dispatch (tool);
+
+	gtk_signal_disconnect_by_func (GTK_OBJECT (tool->platform_dialog),
+				       platform_dialog_close_cb,
+				       tool);
+	
 }
 
 static gboolean
@@ -151,6 +168,8 @@ platform_unsupported_cb (XstTool *tool, XstReportLine *rline)
 			    platform_list_unselect_row_cb, tool);
 	gtk_signal_connect (GTK_OBJECT (tool->platform_dialog), "clicked",
 			    platform_unsupported_clicked_cb, tool);
+	gtk_signal_connect (GTK_OBJECT (tool->platform_dialog), "close",
+			    platform_dialog_close_cb, tool);
 
 	gtk_widget_set_sensitive (tool->platform_ok_button, FALSE);
 	gtk_widget_show (tool->platform_dialog);
@@ -194,6 +213,14 @@ report_finished_cb (XstTool *tool, XstReportLine *rline)
 	return TRUE;
 }
 
+/* --- Utility function --- */
+
+static gint
+xst_tool_compare_platforms (XstPlatform *a, XstPlatform *b)
+{
+	return strcmp (a->name, b->name);
+}
+
 /* --- XstTool --- */
 
 void
@@ -202,11 +229,15 @@ xst_tool_add_supported_platform (XstTool *tool, XstPlatform *platform)
 	g_return_if_fail (tool != NULL);
 	g_return_if_fail (platform != NULL);
 
-	/* TODO: Avoid duplicates. Backend doesn't serve duplicates, but
-	 * we should be paranoid here. */
+	/* Avoid duplicates. Backend shouldn't serve duplicates, but
+	 * we're paranoid here. */
+	g_return_if_fail
+		(g_slist_find_custom (tool->supported_platforms_list, platform,
+		    (GCompareFunc) xst_tool_compare_platforms) == NULL);
 
 	tool->supported_platforms_list =
-		g_slist_append (tool->supported_platforms_list, platform);
+		g_slist_insert_sorted (tool->supported_platforms_list, platform,
+		    (GCompareFunc) xst_tool_compare_platforms);
 }
 
 void
@@ -263,6 +294,7 @@ xst_tool_get_access (XstTool *tool)
 	return root_access != ROOT_ACCESS_NONE;
 }
 
+#if 0
 static void
 timeout_cb (gpointer data)
 {
@@ -276,8 +308,6 @@ timeout_cb (gpointer data)
 		gtk_main_quit ();
 }
 
-#if 0
-
 static void
 set_arrow (XstTool *tool, GtkArrowType arrow)
 {
@@ -311,13 +341,13 @@ report_dispatch (XstTool *tool)
 {
 	GSList *list;
 	XstReportLine *rline;
+#if 0
 	GtkAdjustment *vadj;
 	char *report_text[] = {
 		NULL,
 		NULL
 	};
 
-#if 0
 	if (!GTK_IS_SCROLLED_WINDOW (tool->report_scrolled)) {
 		g_warning ("tool->report_scrolled is not a GtkScrolledWindow\n");
 		tool->report_dispatch_pending = FALSE;
@@ -387,50 +417,63 @@ report_progress_tick (gpointer data, gint fd, GdkInputCondition cond)
 {
 	XstTool *tool;
 	XstReportLine *rline;
-	char c;
+	char buffer [512];
+	int n, i = 0;
 
 	tool = XST_TOOL (data);
 
-#warning FIXME: make this not suck
-	if (!tool->line) {
-		tool->line = g_malloc(1024);
-		tool->line_len = 0;
-		tool->line [0] = '\0';
-	}
+	if (!tool->line)
+		tool->line = g_string_new ("");
 
-	/* NOTE: The read() being done here is inefficient, but we're not
-	 * going to handle any large amount of data */
+	while ((n = read (fd, buffer, sizeof (buffer) - 1)) != 0) {
 
-	if (read (fd, &c, 1) > 0) {
-		if (c == '\n') {
-			/* End of line */
-
-		        /* Report line; add to list */
-			rline = xst_report_line_new_from_string (tool->line);
+		buffer [n] = 0;
+		
+		for (i = 0; (i < n); i++) {
+			char c = buffer [i];
 			
-			if (rline)
-				tool->report_line_list = g_slist_append (tool->report_line_list,
-									 rline);
+			if (c == '\n') {
+				/* End of line */
+				
+				/* Report line; add to list */
+				rline = xst_report_line_new_from_string (tool->line->str);
+				
+				if (rline)
+					tool->report_line_list = g_slist_append (
+						tool->report_line_list, rline);
 
-			tool->line [0] = '\0';
-			tool->line_len = 0;
-		} else {
-			/* Add character to end of current line */
+				g_string_assign (tool->line, "");
 
-			tool->line = g_realloc (tool->line, MAX (tool->line_len + 2, 1024));
-			tool->line [tool->line_len] = c;
-			tool->line_len++;
-			tool->line [tool->line_len] = '\0';
+				if (!tool->report_dispatch_pending){
+					report_dispatch (tool);
+					if (tool->report_finished){
+						i++;
+						goto full_break;
+					}
+				}
+			} else {
+				/* Add character to end of current line */
+				g_string_append_c (tool->line, buffer [i]);
+				
+			}
 		}
 	}
-	else
-	{
+
+ full_break:
+
+	if (n <= 0) {
 		/* Zero-length read; pipe closed unexpectedly */
-		/* FIXME: handle this at the UI level correctly. */
 
 		tool->report_finished = TRUE;
 	}
 
+	if (tool->report_finished) {
+		if (n > 0)
+			tool->xml_document = g_string_new (&buffer [i]);
+		g_string_free (tool->line, TRUE);
+		tool->line = NULL;
+	}
+	
 	if (!tool->report_dispatch_pending)
 		report_dispatch (tool);
 	
@@ -441,10 +484,19 @@ report_progress_tick (gpointer data, gint fd, GdkInputCondition cond)
 	}
 }
 
+static gboolean
+report_window_close_cb (GtkWidget *window, gpointer data)
+{
+	gtk_widget_hide (window);
+	return TRUE;
+}
+
 static void
 report_progress (XstTool *tool, int fd, const gchar *label)
 {
+#if 0
 	gint cb_id;
+#endif	
 
 	tool->timeout_done = FALSE;
 	tool->report_list_visible = FALSE;
@@ -462,6 +514,8 @@ report_progress (XstTool *tool, int fd, const gchar *label)
 	tool->input_id = gtk_input_add_full (fd, GDK_INPUT_READ, report_progress_tick,
 					     NULL, tool, NULL);
 
+	gtk_signal_connect_after (GTK_OBJECT (tool->report_window), "delete-event",
+				  GTK_SIGNAL_FUNC (report_window_close_cb), NULL);
 	gtk_widget_show (tool->report_window);
 	
 	gtk_main ();
@@ -491,8 +545,6 @@ gboolean
 xst_tool_save (XstTool *tool)
 {
 	FILE *f;
-#warning This is a security problem, fix
-	FILE *debug_file;
 	int fd_xml [2], fd_report [2];
 	int t;
 
@@ -502,7 +554,7 @@ xst_tool_save (XstTool *tool)
 
 	g_return_val_if_fail (root_access != ROOT_ACCESS_NONE, FALSE);
 
-	xst_dialog_freeze (tool->main_dialog);
+	xst_dialog_freeze_visible (tool->main_dialog);
 
 	gtk_signal_emit (GTK_OBJECT (tool), xsttool_signals[FILL_XML]);
 
@@ -531,18 +583,17 @@ xst_tool_save (XstTool *tool)
 		close (fd_report [1]);  /* Close writing end of report pipe */
 
 		f = fdopen (fd_xml [1], "w");
-		debug_file = fopen ("/tmp/xst-fe-write", "w");
 
 		xmlDocDump (f, tool->config);
-		xmlDocDump (debug_file, tool->config);
 		
 		fclose (f);
-		fclose (debug_file);
 		
 		report_progress (tool, fd_report [0], _("Updating your system configuration."));
 		close (fd_report [0]);
 
 	} else {
+		gchar *loc_arg;
+
 		/* Child */
 
 		close (fd_xml [1]);	/* Close writing end of XML pipe */
@@ -550,18 +601,21 @@ xst_tool_save (XstTool *tool)
 		dup2 (fd_xml [0], STDIN_FILENO);
 		dup2 (fd_report [1], STDOUT_FILENO);
 
+		loc_arg = (location_id == NULL) ? NULL : "--location";
+
 		if (tool->current_platform)
 			execl (tool->script_path, tool->script_path,
 			       "--set", "--progress", "--report", "--platform",
-			       xst_platform_get_name (tool->current_platform), NULL);
+			       xst_platform_get_name (tool->current_platform),
+			       loc_arg, location_id, NULL);
 		else
 			execl (tool->script_path, tool->script_path, "--set", "--progress",
-			       "--report", NULL);
+			       "--report", loc_arg, location_id, NULL);
 
 		g_error ("Unable to run backend: %s", tool->script_path);
 	}
 
-	xst_dialog_thaw (tool->main_dialog);
+	xst_dialog_thaw_visible (tool->main_dialog);
 	return TRUE;  /* FIXME: Determine if it really worked. */
 }
 
@@ -575,8 +629,7 @@ gboolean
 xst_tool_load (XstTool *tool)
 {
 	int fd [2];
-	int t, len;
-	char *p;
+	int t;
 
 	g_return_val_if_fail (tool != NULL, FALSE);
 	g_return_val_if_fail (XST_IS_TOOL (tool), FALSE);	
@@ -593,13 +646,20 @@ xst_tool_load (XstTool *tool)
 		tool->config = NULL;
 	}
 
-	pipe (fd);
-			  
-	t = fork ();
+	if (location_id == NULL) {
+		pipe (fd);
+		t = fork ();
+	} else {
+		fd[0] = STDIN_FILENO;
+		fd[1] = -1;
+		t = 1;
+	}
 
 	if (t < 0) {
 		g_error ("Unable to fork.");
 	} else if (t) {
+		char buffer [2048];
+		
 		/* Parent */
 
 		close (fd [1]);	/* Close writing end */
@@ -609,27 +669,26 @@ xst_tool_load (XstTool *tool)
 		 * mechanism. Let's just load it all into memory, then. Also, refusing
 		 * enormous documents can be considered a plus. </dystopic> */
 
-		report_progress (tool, fd [0], _("Scanning your system configuration."));
+		if (location_id == NULL)
+			report_progress (tool, fd [0], _("Scanning your system configuration."));
 
 		if (tool->run_again)
 			return TRUE;
 
-		p = g_malloc (102400);
+		if (tool->xml_document == NULL)
+			tool->xml_document = g_string_new ("");
+		
 		fcntl(fd [0], F_SETFL, 0);  /* Let's block */
-		len = 0; 
-		for (len = 0; (t = read (fd [0], p + len, 102399 - len)); len += t)
-			;
 
-		if (len < 1 || len == 102399) {
-			g_free (p);
-		} else {
-			p = g_realloc (p, len + 1);
-			*(p + len) = 0;
-
-			tool->config = xmlParseDoc (p);
-			g_free (p);
-			close (fd [0]);
+		while ((t = read (fd [0], buffer, sizeof (buffer)-1)) != 0){
+			buffer [t] = 0;
+			g_string_append (tool->xml_document, buffer);
 		}
+		
+		tool->config = xmlParseDoc (tool->xml_document->str);
+		g_string_free (tool->xml_document, TRUE);
+		tool->xml_document = NULL;
+		close (fd [0]);
 	} else {
 		/* Child */
 
@@ -639,12 +698,14 @@ xst_tool_load (XstTool *tool)
 		if (tool->current_platform)
 			execl (tool->script_path, tool->script_path,
 			       "--get", "--progress", "--report", "--platform",
-			       xst_platform_get_name (tool->current_platform), NULL);
+			       xst_platform_get_name (tool->current_platform),
+			       NULL);
 		else
 			execl (tool->script_path, tool->script_path, "--get", "--progress",
 			       "--report", NULL);
 
-		g_error ("Unable to run backend: %s", tool->script_path);
+		g_error ("Unable to run backend %s: %s", tool->script_path,
+			 g_strerror (errno));
 	}
 	
 	if (tool->config)
@@ -673,15 +734,17 @@ xst_tool_load_try (XstTool *tool)
 }
 
 void
-xst_tool_main (XstTool *tool)
+xst_tool_main (XstTool *tool, gboolean no_main_loop)
 {
-	xst_dialog_freeze (tool->main_dialog);
+	xst_dialog_freeze_visible (tool->main_dialog);
 	gtk_widget_show (GTK_WIDGET (tool->main_dialog));
 
 	xst_tool_load_try (tool);
 
-	xst_dialog_thaw (tool->main_dialog);
-	gtk_main ();
+	xst_dialog_thaw_visible (tool->main_dialog);
+
+	if (!no_main_loop)
+		gtk_main ();
 }
 
 static void
@@ -765,6 +828,8 @@ xst_tool_type_init (XstTool *tool)
 
 	tool->glade_common_path  = g_strdup_printf ("%s/common.glade", INTERFACES_DIR);
 
+	tool->etspecs_common_path = g_strdup (ETSPECS_DIR);
+
 	xml = xst_tool_load_glade_common (tool, "report_window");
 
 	tool->report_window     = glade_xml_get_widget (xml, "report_window");
@@ -797,8 +862,8 @@ xst_tool_type_init (XstTool *tool)
 #endif
 
 	xml = xst_tool_load_glade_common (tool, "platform_dialog");
-	tool->platform_dialog = glade_xml_get_widget (xml, "platform_dialog");
-	tool->platform_list = glade_xml_get_widget (xml, "platform_list");
+	tool->platform_dialog    = glade_xml_get_widget (xml, "platform_dialog");
+	tool->platform_list      = glade_xml_get_widget (xml, "platform_list");
 	tool->platform_ok_button = glade_xml_get_widget (xml, "platform_ok_button");
 }
 
@@ -852,13 +917,11 @@ xst_tool_construct (XstTool *tool, const char *name, const char *title)
 }
 
 XstTool *
-xst_tool_new (const char *name, const char *title)
+xst_tool_new (void)
 {
 	XstTool *tool;
-	g_return_val_if_fail (name != NULL, NULL);
 
 	tool = XST_TOOL (gtk_type_new (XST_TYPE_TOOL));
-	xst_tool_construct (tool, name, title);
 
 	return tool;
 }
@@ -937,33 +1000,124 @@ xst_tool_reset_report_hooks (XstTool *tool)
 		((XstReportHook *) list->data)->invoked = FALSE;
 }
 
-XstTool *
-xst_tool_init (const char *name, const char *title, int argc, char *argv [], const poptOption options)
+/**
+ * xst_fool_the_linker:
+ * @void: 
+ * 
+ * We need to keep the symbol for the create image widget function
+ * so that libglade can find it to create the icons.
+ **/
+void xst_fool_the_linker (void);
+void
+xst_fool_the_linker (void)
 {
-	GtkWidget *d;
+	xst_ui_create_image_widget (NULL, NULL, NULL, 0, 0);
+}
 
-	g_return_val_if_fail (name != NULL, NULL);
-	g_return_val_if_fail (title != NULL, NULL);
+static void
+authenticate (gchar *exec_path)
+{
+	GtkWidget *error_dialog;
+	gchar *password;
+	gint result;
+
+	for (;;)
+	{
+		result = xst_su_get_password (&password);
+
+		if (result < 0)
+			exit (0);
+		else if (result == 0)
+			return;
+
+		/* If successful, the following never returns */
+
+		xst_su_run_with_password (exec_path, password);
+
+		if (strlen (password))
+			memset (password, 0, strlen (password));
+
+		error_dialog = gnome_error_dialog (_("The password you entered is invalid."));
+		gnome_dialog_run_and_close (GNOME_DIALOG (error_dialog));
+	}
+}
+
+static void
+try_show_usage_warning (void)
+{
+	gchar *key;
+	gboolean value;
+	gchar *warning = g_strdup_printf(_("Welcome to the %s prerelease of the "
+		  "Ximian Setup Tools.\n\n"
+		  "This is still a work in progress, and so it may have serious bugs.\n"
+		  "Due to the nature of these tools, bugs may render your computer\n"
+		  "PRACTICALLY USELESS, costing time, effort and sanity points.\n\n"
+		  "You have been warned. Thank you for trying out this prerelease of\n"
+		  "the Ximian Setup Tools!\n\n"
+		  "--\nThe Ximian Setup Tools team"), VERSION);
+
+	key = g_strjoin ("/", XST_CONF_ROOT, "global", "previously-run-" VERSION, NULL);
+
+	value = gnome_config_get_bool (key);
+
+	if (!value)
+	{
+		GnomeDialog *dialog;
+		GtkWidget *w0, *w1, *w2;
+
+		dialog = GNOME_DIALOG (gnome_warning_dialog (warning));
+		w0 = gtk_container_children (GTK_CONTAINER (GTK_BIN (GTK_BIN (dialog)->child)->child))->data;
+		w1 = GTK_WIDGET (gtk_check_button_new_with_label (_("Don't show me this again")));
+		gtk_widget_ref (w1);
+		w2 = GTK_WIDGET (gtk_alignment_new (1.0, 0.0, 0.0, 0.0));
+		gtk_container_add (GTK_CONTAINER (w2), w1);
+		gtk_box_pack_end (GTK_BOX (w0), w2, TRUE, TRUE, 0);
+		gtk_widget_show (w1);
+		gtk_widget_show (w2);
+
+		gnome_dialog_run_and_close (dialog);
+
+		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (w1)))
+		{
+			gnome_config_set_bool (key, TRUE);
+			gnome_config_sync ();
+		}
+
+		gtk_widget_unref (w1);
+	}
+
+	g_free (warning);
+
+	g_free (key);
+}
+
+void
+xst_init (const gchar *app_name, int argc, char *argv [], const poptOption options)
+{
+	struct poptOption xst_options[] =
+	{
+		{ "location", '\0', POPT_ARG_STRING, &location_id, 0,
+		  N_("	Edit configuration in the location LOCATION."), N_("LOCATION") },
+		
+		{NULL, '\0', 0, NULL, 0}
+	};
 
 #ifdef ENABLE_NLS
 	bindtextdomain (PACKAGE, GNOMELOCALEDIR);
 	textdomain (PACKAGE);
 #endif
 
-	/* This is to fool the linker: do not delete, even if it doesn't
-	 * make sense. Arturo Espinosa <arturo@ximian.com>
-	 */
-	xst_ui_create_image_widget (NULL, NULL, NULL, 0, 0);
+        gnomelib_register_popt_table (xst_options, "general XST options");
 
 	if (options == NULL) {
-		gnome_init (name, VERSION, argc, argv);
+		gnome_init (app_name, VERSION, argc, argv);
 	} else {
 		poptContext ctx;
 		GList *args_list = NULL;
 		char **args;
 		gint i;
 		
-		gnome_init_with_popt_table (name, VERSION, argc, argv, options, 0, &ctx);
+		gnome_init_with_popt_table (app_name, VERSION, argc, argv, options, 0, &ctx);
 
 		args = (char**) poptGetArgs(ctx);
 	
@@ -984,18 +1138,9 @@ xst_tool_init (const char *name, const char *title, int argc, char *argv [], con
 		root_access = ROOT_ACCESS_SIMULATED;
 #endif
 	} else {
-		d = gnome_ok_cancel_dialog (_("You need full administration privileges (i.e. root)\n"
-					      "to run this configuration tool. You can acquire\n"
-					      "such privileges by running it from the GNOME Control\n"
-					      "Center or issuing an \"su\" command in a shell.\n\n"
-					      "You will be unable to make any changes. Continue anyway?"), 
-					    NULL, NULL);
-
-		if (gnome_dialog_run_and_close (GNOME_DIALOG (d)))
-			exit (1);
-		
+		authenticate (argv [0]);
 		root_access = ROOT_ACCESS_NONE;
 	}
 
-	return xst_tool_new (name, title);
+	try_show_usage_warning ();
 }
